﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ORM04
{
    class AttributeMapping
    {
        private static Dictionary<Type, EntityMapping> entityMappings = new Dictionary<Type, EntityMapping>();

        public static EntityMapping Get<T>()
        {
            return Get(typeof(T));
        }

        public static EntityMapping Get(Type type)
        {
            EntityMapping entityMapping;
            if (entityMappings.TryGetValue(type, out entityMapping) == false)
            {
                entityMapping = CreateMapping(type);
                entityMappings.Add(type, entityMapping);
            }
            return entityMapping;
        }

        private static EntityMapping CreateMapping(Type type)
        {
            EntityMapping entityMapping = new EntityMapping()
            {
                EntityType = type
            };
            string tableName = string.Empty;

            var properties = type.GetProperties();

            TableAttribute[] tableAttrs = (TableAttribute[])type.GetCustomAttributes(typeof(TableAttribute), true);
            if (tableAttrs.Length > 0)
            {
                tableName = tableAttrs[0].Name;
            }
            else
            {
                tableName = type.Name;
            }


            List<MemberMapping> members = new List<MemberMapping>();
            for (int i = 0; i < properties.Length; i++)
            {
                var pi = properties[i];

                var idAttrs = (IdAttribute[])pi.GetCustomAttributes(typeof(IdAttribute), true);
                bool isPrimaryKey = idAttrs.Length > 0;

                var attrs = (ColumnAttribute[])pi.GetCustomAttributes(typeof(ColumnAttribute), true);
                if (attrs.Length > 0)
                {
                    var m = new MemberMapping()
                    {
                        ColumnName = attrs[0].Name,
                        IsDbGenerated = attrs[0].IsGenerated,
                        Member = pi,
                        IsPrimaryKey = isPrimaryKey
                    };
                    members.Add(m);
                }
            }

            entityMapping.TableName = tableName;
            entityMapping.Members = members;
            entityMapping.PrimaryKey = members.Where(c => c.IsPrimaryKey).ToList();
            return entityMapping;

        }
    }
}