﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ORM04
{
    class DbSqlBuilder
    {
        public string BuildInsertSql(EntityMapping entityMapping, List<string> columnNames)
        {
            StringBuilder sql = new StringBuilder();
            sql.Append("INSERT INTO ").Append(entityMapping.TableName).Append(" (");

            var columns = entityMapping.Members.Where(m => m.IsDbGenerated == false && columnNames.Contains(m.ColumnName,StringComparer.OrdinalIgnoreCase)).ToArray();

            for (int i = 0; i < columns.Length; i++)
            {
                var member = columns[i];
                if (i > 0)
                    sql.Append(",");

                sql.Append(member.ColumnName);
            }

            sql.Append(") VALUES (");
            for (int i = 0; i < columns.Length; i++)
            {
                var member = columns[i];
                if (i > 0)
                    sql.Append(",");

                sql.Append("@").Append(member.ColumnName);
            }

            sql.Append(")");

            return sql.ToString();
        }
        public string BuildUpdateSql(EntityMapping entityMapping, List<string> updateColumns,List<string> whereColumns)
        {
            StringBuilder sql = new StringBuilder();
            sql.Append("UPDATE ").Append(entityMapping.TableName).Append(" SET ");

            var columns = entityMapping.Members.Where(m => updateColumns.Contains(m.ColumnName, StringComparer.OrdinalIgnoreCase)).ToArray();
            var wheres = entityMapping.Members.Where(m => whereColumns.Contains(m.ColumnName, StringComparer.OrdinalIgnoreCase)).ToArray();

            for (int i = 0; i < columns.Length; i++)
            {
                var member = columns[i];
                if (i > 0)
                    sql.Append(",");

                sql.Append(member.ColumnName).Append("=").Append("@").Append(member.ColumnName);
            }

            sql.Append(" WHERE ");
            for (int i = 0; i < wheres.Length; i++)
            {
                var member = wheres[i];
                if (i > 0)
                    sql.Append(",");

                sql.Append(member.ColumnName).Append("=").Append("@p").Append(member.ColumnName);
            }

            return sql.ToString();
        }
        public string BuildDeleteSql(EntityMapping entityMapping, List<string> whereColumns)
        {
            StringBuilder sql = new StringBuilder();
            sql.Append("DELETE FROM ").Append(entityMapping.TableName).Append(" WHERE ");

            var wheres = entityMapping.Members.Where(m => whereColumns.Contains(m.ColumnName, StringComparer.OrdinalIgnoreCase)).ToArray();

            for (int i = 0; i < wheres.Length; i++)
            {
                var member = wheres[i];
                if (i > 0)
                    sql.Append(",");

                sql.Append(member.ColumnName).Append("=").Append("@").Append(member.ColumnName);
            }

            return sql.ToString();
        }
        public string BuildSelectSql(EntityMapping entityMapping,string strWhere, string orderBy)
        {
            StringBuilder sql = new StringBuilder();
            sql.Append("SELECT * FROM ").Append(entityMapping.TableName);

            if (string.IsNullOrEmpty(strWhere) == false)
                sql.Append(" WHERE ").Append(strWhere);

            if (string.IsNullOrEmpty(orderBy) == false)
                sql.Append(" Order By ").Append(orderBy);
    
            return sql.ToString();
        }
    }
}